#include <stdio.h>
#include <EEPROM.h>

enum AlarmState {
  NOT_SET = 0, SET = 1
};

class Alarm {
private:
  int hour;
  int min;
  int alarmState;
  int index;
  int alCursor;

public:
  Alarm(int index) {
    this->index = index * 4;
    uint32_t val = ((uint32_t)EEPROM.read(this->index))
                 | ((uint32_t)EEPROM.read(this->index + 1)) << 8 
                 | ((uint32_t)EEPROM.read(this->index + 2)) << 16 
                 | ((uint32_t)EEPROM.read(this->index + 3)) << 24;
    if (val == 0xFFFFFFFF) {
      this->hour = 24;
      this->min = 0;
      this->alarmState = NOT_SET;
      return;
    }
    this->min = val % 100;
    val /= 100;
    this->hour = val % 100;
    val /= 100;
    this->alarmState = val % 10;
    if (this->alarmState == NOT_SET) {
      this->hour = 24;
    }
  }

  void save() {
    uint32_t val = 1000000 + this->alarmState * 100000 + this->hour * 100 + this->min;  
    EEPROM.update(this->index, (uint8_t) (val >> 24 & 0xFF));
    EEPROM.update(this->index + 1, (uint8_t) (val >> 16 & 0xFF));
    EEPROM.update(this->index + 2, (uint8_t) (val >> 8 & 0xFF));
    EEPROM.update(this->index + 3, (uint8_t) (val & 0xFF));            
  }
  
  int getHour() {
    return this->hour;
  }

  int incHour() {
    if (this->hour == 24) {
      this->hour = 0;
    } else {
      this->hour++;      
    }
  }

  int decHour() {
    if (this->hour == 0) {
      this->hour = 24;
    } else {
      this->hour--;
    }
  }

  int getMin() {
    return this->min;
  }

  int incMin() {
    if (this->min == 60) {
      this->min = 0;
    } else {
      this->min++;      
    }
  }

  int decMin() {
    if (this->min == 0) {
      this->min = 60;
    } else {
      this->min--;
    }
  }

  int getAlCursor() {
    return this->alCursor;
  }

  int incAlCursor() {
    this->alCursor++;
  }

  int decAlCursor() {
    this->alCursor--;
  }

  int initAlarm() {
    this->alCursor = 0;
  }

  void getAlarm(char* alarm) {
    sprintf(alarm, "%d:%d", this->hour, this->min);
  }
  
  void setAlarm() {
    this->alarmState = SET;
    save();
  }

  void deleteAlarm() {
    this->hour = 24;
    this->min = 0;
    this->alarmState = NOT_SET;
    save();
  }

  int getAlarmState() {
    return alarmState;    
  }

};


class Settings {
private:
  Alarm alarms[4] = {
    Alarm(0),
    Alarm(1),
    Alarm(2),
    Alarm(3)
  };
  int cursor;

public:
  Settings() {
    for (int i = 0; i < 16; i++) {
      EEPROM.write(i, 255);
    }
    this->cursor = 0;
  }

  int getCursor() {
    return this->cursor;
  }

  void incCursor() {
    if (this->cursor < 7) {
      this->cursor++;      
    }
  }

  void decCursor() {
    if (this->cursor > 0) {
      this->cursor--;
    }    
  }

  void resetCursor() {
    this->cursor = 0;
  }

  Alarm* getAlarm(int idx) {
    return &(alarms[idx]);
  }  
};
